#Clear workspace
rm(list = ls())

library(maps)
library(boot)
library(RColorBrewer)
library(easyVerification)
library(SpecsVerification)
library(s2dv)
library(abind)
#library(multiApply)
#library(ncdf4)
library(startR)
#library(plyr)

retrieve <- FALSE

exp <- 'a3w5'
vari <- 'tas'
mod <- 'Amon'
grid <- '1x1'
t1 <- 1
t2 <- 60 #132
yr1 <- '1980'
yr2 <- '1982'
lats <- c(-10, 10)
lons <- c(0, 20) #359.9)
membs <- c('r1i1p1f1','r2i1p1f1','r3i1p1f1','r4i1p1f1','r5i1p1f1','r6i1p1f1','r7i1p1f1','r8i1p1f1','r9i1p1f1','r10i1p1f1')
nmemb <- length(membs)

#--------------------------
# Step1: Load CMIP6 DCPP
#--------------------------
repos <- paste0('/esarchive/exp/ecearth/', exp, '/diags/DCPP/EC-Earth-Consortium/EC-Earth3/dcppA-hindcast/$memb$/',
                mod, '/$var$/', grid, '/v*/$var$_', mod, 
                '_EC-Earth3_dcppA-hindcast_s$sdate$1101-$memb$_', grid, 
                '_$chunk$.nc')

data <- Start(dat = repos,
              var = vari,
              memb = membs,
              sdate = paste(seq(yr1, yr2)),
              chunk = 'all',
              chunk_depends = 'sdate',
              time = c(t1:t2),
              time_across = 'chunk',
              merge_across_dims = TRUE,
              latitude = values(list(lats[1], lats[2])),
              longitude = values(list(lons[1], lons[2])),
#              latitude = 'all',
              latitude_reorder = Sort(),
#              longitude = 'all',
              longitude_reorder = CircularSort(0, 360),
              synonims = list(longitude = c('lon', 'longitude'), 
                              latitude = c('lat', 'latitude')),
#              transform = CDORemapper, transform_extra_cells = 2, transform_params = list(grid = 'r360x180', method = 'con', crop = FALSE), transform_vars = c('lat','lon'),
              return_vars = list(time = c('sdate', 'chunk'), 
                                 latitude = NULL, longitude = NULL),
              retrieve = retrieve)
#39.1Gb

attr(data, 'Dimensions')
#      dat       var      memb     sdate      time  latitude longitude 
#        1         1        10         1        36       180       360 

lat_exp <- as.vector(attr(data, 'Variables')$dat1$latitude) 
lon_exp <- as.vector(attr(data, 'Variables')$dat1$longitude)

#--------------------------
# Step2: Load recon
#--------------------------
recon <- "a3t4"
grid_recon <- "r360x180"
membs_recon <- c('r1i1p1f1','r2i1p1f1','r3i1p1f1','r4i1p1f1','r5i1p1f1')
path_recon <- paste0('/esarchive/exp/ecearth/', recon, '/diags/CMIP/EC-Earth-Consortium/EC-Earth3-LR/piControl/$memb$/' , mod, '/$var$/',
               grid_recon, '/v*/$var$_', mod, '_EC-Earth3-LR_piControl_$memb$_', grid, '_$chunk$.nc')

data_recon <- Start(dat = path_recon,
              var = vari,
              memb = membs_recon,
              chunk = 'all',
              latitude = 'all',
              latitude_reorder = Sort(),
              longitude = 'all',
              longitude_reorder = CircularSort(0, 360),
              time = t1:t2,
              time_across = 'chunk',
              merge_across_dims = TRUE,
              transform = CDORemapper, transform_extra_cells = 2, 
              transform_params = list(grid = '/esarchive/exp/ecearth/a3w5/diags/DCPP/EC-Earth-Consortium/EC-Earth3/dcppA-hindcast/r1i1p1f1/Amon/tas/1x1/v20210910/tas_Amon_EC-Earth3_dcppA-hindcast_s19601101-r1i1p1f1_1x1_196011-196110.nc', 
                                      method = 'con'), 
              transform_vars = c('lat','lon'),
              synonims = list(longitude = c('lon', 'longitude'),
                              latitude = c('lat', 'latitude')),
              return_vars = list(time = 'chunk', latitude = NULL, longitude = NULL),
              retrieve = retrieve)

attr(data_recon, 'Dimensions')
#      dat       var      memb  latitude longitude      time 
#        1         1         5       180       360        36 

#Hist to hind
#recon_sdates <- gsub("-", "", paste(seq(as.Date("1960/11/01"), as.Date("2021/11/01"), "years"), sep = "-"))
data_sdates <- strftime(seq(as.Date(data_dates[1, 1, 1]), as.Date(data_dates[dim(data_dates)['sdate'], 1, 1]), "years"), "%Y%m")
data_recon_hc <- Reorder(Histo2Hindcast(data_recon, sdatesin = strftime(recon_dates[1], "%Y%m"), sdatesout = data_sdates, nleadtimesout = dim(data)['time'], sdate_dim = "sdate", ftime_dim = "time"),
                         c('memb', 'sdate', 'time'))


#--------------------------
# Step3: Load obs
#--------------------------
path_obs <- '/esarchive/recon/ecmwf/era5/monthly_mean/$var$_f1h/$var$_$file_date$.nc'
#path_obs <- '/esarchive/obs/ukmo/hadisst_v1.1/monthly_mean/$var$/$var$_$file_date$.nc'
time_obs <- attr(data, 'Variables')$common$time #[sdate, time]
data_obs <- Start(dat = path_obs,
              var = vari,
              file_date = unique(format(time_obs, '%Y%m')), #paste(seq(yr1, yr2)),
              time = values(time_obs),
              time_across = 'file_date',
              merge_across_dims = TRUE,
              split_multiselected_dims = TRUE,
              latitude = values(list(lats[1], lats[2])),
              longitude = values(list(lons[1], lons[2])),
              latitude_reorder = Sort(),
              longitude_reorder = CircularSort(0, 360),
              transform = CDORemapper, transform_extra_cells = 2, 
              transform_params = list(grid = '/esarchive/exp/ecearth/a3w5/diags/DCPP/EC-Earth-Consortium/EC-Earth3/dcppA-hindcast/r1i1p1f1/Amon/tas/1x1/v20210910/tas_Amon_EC-Earth3_dcppA-hindcast_s19601101-r1i1p1f1_1x1_196011-196110.nc',  
                                      method = 'con'),
              transform_vars = c('latitude', 'longitude'),
              synonims = list(longitude = c('lon', 'longitude'),
                              latitude = c('lat', 'latitude')),
              return_vars = list(time = c('file_date'), 
                                 latitude = NULL, longitude = NULL),
              retrieve = retrieve)
attr(data_obs, 'Dimensions')
#      dat       var  latitude longitude     sdate      time 
#        1         1       181       360         3       132 

# #Interpolate
# if(interp == 'TRUE'){
# print("Interpolate Data")
# data_int = s2dverification::CDORemap(data, lon_cmip6, lat_cmip6, 'r360x180', 'con')
# data = data_int$data_array
# lat_cmip6 = data_int$lat
# lon_cmip6 = data_int$lon
# }

if (isTRUE(retrieve)) {
  #Annual means
  data2 <- drop(Reorder(Season(data, time_dim = 'time', monini = 11, 
                               moninf = 1, monsup = 12, method = mean, na.rm = TRUE),
                c(2, 3, 4, 5, 1, 6, 7)))
  dim(data2)

} else if (isFALSE(retrieve)) {
  
  fun <- function(data, data_obs) {
    # data: [sdate, time, memb]
    # data: [sdate, time]
#print('in fun!!')
    data_yrs <- apply(data, c(1, 3), s2dv:::.Season, monini = 11, moninf = 1, monsup = 12, method = mean, na.rm = TRUE)
    data_obs_yrs <- apply(data_obs, 1, s2dv:::.Season, monini = 11, moninf = 1, monsup = 12, method = mean, na.rm = TRUE)
#    data_yr <- s2dv:::.Season(data, monini = 11, moninf = 1, 
#                              monsup = 12, method = mean, na.rm = TRUE)
#    data_obs_yr <- s2dv:::.Season(data_obs_yr, monini = 11, moninf = 1,
#                              monsup = 12, method = mean, na.rm = TRUE)
#print('Season done!!')
#print(dim(data_yrs)) #[time, sdate, memb]
#print(dim(data_obs_yrs))
#print(summary(data_yrs))
    # Clim
    data_yrs <- s2dv::InsertDim(data_yrs, 3, 1, 'dat')
    data_obs_yrs <- s2dv::InsertDim(s2dv::InsertDim(data_obs_yrs, 3, 1, 'dat'), 4, 1, 'memb')
#print('Insert done!!')
#print(dim(data_yrs))
#print(dim(data_obs_yrs))
    clim_data <- array(dim = c(time = dim(data_yrs)[1]))
    clim_data_obs <- array(dim = c(time = dim(data_yrs)[1]))
    for (tt in 1:dim(data_yrs)[1]) {
      data_tmp <- ClimProjDiags::Subset(data_yrs, 1, tt, drop = 'selected')
      data_obs_tmp <- ClimProjDiags::Subset(data_obs_yrs, 1, tt, drop = 'selected')
      tmp <- s2dv:::.Clim(data_tmp, data_obs_tmp, 
                          time_dim = 'sdate', method = "clim", dat_dim = c('dat', 'memb'), 
                          memb_dim = 'memb', memb = FALSE, na.rm = TRUE)
      clim_data[tt] <- tmp$clim_exp
      clim_data_obs[tt] <- tmp$clim_obs
    }
#print('Clim done!!')
#print(dim(clim_data))
#print(dim(clim_data_obs))
#    #Calculate anomalies & ensemble mean
#    data_yrs_anom <- Ano(data_yrs, clim_data$clim_exp)
#dim(data_yrs_anom)
#    data_yrs_anom_ens <- s2dv::MeanDims(data_yrs_anom, 'memb') 

    # diff (clim - obs)
    diff <- clim_data - clim_data_obs
    return(diff)
#    return(list(clim_exp = clim_data, clim_obs = clim_data_obs))
  }
  
#  max_jobs <- length(membs)
#  chunks <- list(memb = as.numeric(attr(data,'Dimensions')['memb']),
#                 sdate = 5)
  
  step <- Step(fun = fun,
               use_libraries = c('s2dv', 'ClimProjDiags'),
               target_dims = list(data = c('sdate', 'time', 'memb'), 
                                  data_obs = c('sdate', 'time')), 
#               output_dims = list(clim_exp = c('time'), clim_obs = c('time')))
               output_dims = 'time')
  
  wf <- AddStep(inputs = list(data, data_obs), 
                step_fun = step)
  
 # local
  output <- Compute(workflow = wf, #wf$clim_exp,
                    chunks = list(latitude = 2))

 # Nord4
  output <- Compute(workflow = wf,
                    chunks = chunks,
                    threads_load = 1,
                            threads_compute = ncores,
                            cluster = list(queue_host = 'nord3',
                                           queue_type = 'lsf',
                                           temp_dir = '/gpfs/scratch/bsc32/bsc32135/startR_hpc/',
                                           cores_per_job = ncores,
                                           job_wallclock = '02:00',
                                           max_jobs = max_jobs,
                                           extra_queue_params = list('#BSUB -q bsc_es -M 3000'), # node exclusive: #BSUB -x
                                           bidirectional = FALSE,
                                           polling_period = 10),
                            ecflow_suite_dir = '/home/Earth/rfernand/startR_local/',
                            wait = TRUE)$output1
  
}


# Plotting
diff_clim_obs <- drop(output$output1)
lons <- as.vector(attr(data, 'Variables')$common$longitude)
lats <- as.vector(attr(data, 'Variables')$common$latitude)
chosen_yr <- 1 
color_range <- c(-max(abs(range(diff_clim_obs[chosen_yr, , ]))), max(abs(range(diff_clim_obs[chosen_yr, , ]))))

s2dv::PlotEquiMap(diff_clim_obs[1, , ], lon = lons, lat = lats, toptitle = paste0('SST bias (to ', obs, ') ',exp, ', ', vari, ', yr', chosen_yr), 
                  filled.continents = FALSE, bar_limits = color_range)

#=============================================

sdate <- c(as.numeric(yr1):as.numeric(yr2))
ensem <- c(1:nmemb)
time <- c(t1:t2)
latitude <- lat_exp
longitude <- lon_exp



# Create netcdf file:
filename <- paste0('dcpp_', vari, '_latlon_r1-10.nc')
flcomment <-  paste0('DCPP latlon North Atl.') 
# Create netcdf file including yearly anomalies:
units= 'degrees'
fillvalue=-1
dim1 = ncdim_def( "member","yr", ensem)
dim2 = ncdim_def( "sdate","yr", sdate)
dim3 = ncdim_def( "time","yr", time)
dim4 = ncdim_def( "latitude","degrees_north", as.vector(lat_cmip6))
dim5 = ncdim_def( "longitude","degrees_east", as.vector(lon_cmip6))
var1 = ncvar_def( vari,units,list(dim5,dim4,dim1,dim2,dim3),fillvalue,flcomment)
nc.file = nc_create(filename,list(var1))
ncvar_put(nc.file,var1,data2)
nc_close(nc.file)



